delsum-lib 0.2.0

Library with most functionality for the delsum crate
docs.rs failed to build delsum-lib-0.2.0
Please check the build logs for more information.
See Builds for ideas on how to fix a failed build, or Metadata for how to configure docs.rs builds.
If you believe this is docs.rs' fault, open an issue.
Visit the last successful build: delsum-lib-1.0.0

delsum-lib

This is the library part of delsum, where delsum is just a small binary frontend for this library.