FletcherBuilder

Struct FletcherBuilder 

pub struct FletcherBuilder<Sum: Modnum> { /* private fields */ }
A builder for a fletcher.

One can use it for specifying a fletcher algorithm, which can be used for checksumming.

Example:

let adler32 = Fletcher::<u32>::with_options()
    .width(32)
    .init(1)
    .modulus(65521)
    .check(0x091e01de)
    .name("adler32")
    .build()
    .is_ok();

Implementations

Source

impl<S: Modnum> FletcherBuilder<S>

pub fn width(&mut self, w: usize) -> &mut Self

Sets the width of the type (both sums included, must be even, mandatory)

pub fn modulus(&mut self, m: S) -> &mut Self

Sets the modulus of both sums (mandatory)

pub fn init(&mut self, i: S) -> &mut Self

Sets the initial value

Contains one value for the regular sum.

pub fn addout(&mut self, o: S::Double) -> &mut Self

Sets a value that gets added after the checksum is finished

Contains separate values for both sums, the cumulative one is bitshifted

pub fn swap(&mut self, s: bool) -> &mut Self

Normally, the cumulative sum is saved on the higher bits and the normal sum in the lower bits. Setting this option to true swaps the positions.

pub fn inendian(&mut self, e: Endian) -> &mut Self

The endian of the words of the input file

pub fn wordsize(&mut self, n: usize) -> &mut Self

The number of bits in a word of the input file

pub fn outendian(&mut self, e: Endian) -> &mut Self

The endian of the checksum

pub fn signedness(&mut self, s: Signedness) -> &mut Self

The signedness of the input words

pub fn check(&mut self, c: S::Double) -> &mut Self

Checks whether c is the same as the checksum of “123456789” on creation

pub fn name(&mut self, n: &str) -> &mut Self

A name to be displayed

pub fn build(&self) -> Result<Fletcher<S>, CheckBuilderErr>

Returns the Fletcher object after verifying correctness