A simple modular sum over bytes (i.e. bytes.sum() % modulus)
There are a number of parameters:
- width: The number of bits in the sum type, at most 64
- modulus: The sum is taken modulo this number
- init: The initial number
- check: The checksum of “123456789” (optional, gets checked at construction)
- name: An optional name that gets used for display purposes
Note that a parameter to add at the end is not needed, since it is equivalent to init.
Structs
- ModSum
- A Modsum checksum algorithm.
- ModSum
Builder - A builder to set the various parameters for the modsum algorithm.
Functions
- reverse_
modsum - Find the parameters of a modsum algorithm.