pub fn reverse_modsum<'a>(
spec: &ModSumBuilder<u64>,
chk_bytes: &[(&'a [u8], Vec<u8>)],
verbosity: u64,
extended_search: bool,
) -> impl Iterator<Item = Result<ModSum<u64>, CheckReverserError>> + use<'a>Expand description
Find the parameters of a modsum algorithm.
spec contains the known parameters of the algorithm (by setting the corresponding values in the builder).
chk_bytes are pairs of files and their checksums.
verbosity makes the function output what it is doing
The width parameter of the builder has to be set.